// @HEADER
// ************************************************************************
//
//        Piro: Strategy package for embedded analysis capabilitites
//                  Copyright (2010) Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Andy Salinger (agsalin@sandia.gov), Sandia
// National Laboratories.
//
// ************************************************************************
// @HEADER

#ifndef PIRO_ROL_OBSERVERBASE_HPP
#define PIRO_ROL_OBSERVERBASE_HPP

#include <string>

#include "Thyra_VectorBase.hpp"
#include "Thyra_DefaultMultiVectorProductVector.hpp"

namespace Piro {

template <typename Scalar>
class ROL_ObserverBase {
public:

  virtual void observeSolution(
    double stamp, const Thyra::VectorBase<Scalar>& nonOverlappedSolution,
    const Teuchos::Ptr<const Thyra::MultiVectorBase<Scalar>>& nonOverlappedSolution_dxdp,
    const Teuchos::Ptr<const Thyra::VectorBase<Scalar>>& nonOverlappedSolutionDot,
    const Teuchos::Ptr<const Thyra::VectorBase<Scalar>>& nonOverlappedSolutionDotDot);

  virtual void observeSolution(
      double stamp, const Thyra::MultiVectorBase<Scalar>& nonOverlappedSolution, 
      const Teuchos::Ptr<const Thyra::MultiVectorBase<Scalar>>& nonOverlappedSolution_dxdp);

  virtual void parameterChanged(
      const std::string& param);

  virtual void parametersChanged();

  virtual void observeResponse(int iter);

  virtual ~ROL_ObserverBase() {}
};

template <typename Scalar>
void
ROL_ObserverBase<Scalar>::observeSolution(
    double stamp, const Thyra::VectorBase<Scalar>& /*nonOverlappedSolution*/,
    const Teuchos::Ptr<const Thyra::MultiVectorBase<Scalar>>& /*nonOverlappedSolution_dxdp*/,
    const Teuchos::Ptr<const Thyra::VectorBase<Scalar>>& /*nonOverlappedSolutionDot*/,
    const Teuchos::Ptr<const Thyra::VectorBase<Scalar>>& /*nonOverlappedSolutionDotDot*/)
{
  // Nothing to do by default
}

template <typename Scalar>
void
ROL_ObserverBase<Scalar>::observeSolution(
      double stamp, const Thyra::MultiVectorBase<Scalar>& /*nonOverlappedSolution*/, 
      const Teuchos::Ptr<const Thyra::MultiVectorBase<Scalar>>& /*nonOverlappedSolution_dxdp*/)
{
  // Nothing to do by default
}

template <typename Scalar>
void
ROL_ObserverBase<Scalar>::parameterChanged(
      const std::string& param)
{
  // Nothing to do by default
}

template <typename Scalar>
void
ROL_ObserverBase<Scalar>::parametersChanged()
{
  // Nothing to do by default
}

template <typename Scalar>
void
ROL_ObserverBase<Scalar>::observeResponse(
      int iter)
{
  // Nothing to do by default
}

} // namespace Piro

#endif /* PIRO_ROL_OBSERVERBASE_HPP */
